library jaguar_mux.jaguar_mux;

import 'package:jaguar/jaguar.dart';

import 'mux/mux.dart';

part 'group.dart';
part 'route.dart';

class MuxBuilder {
  final List<Route> _routes = [];

  List<Route> get routes => new List.unmodifiable(_routes);

  final List<InterceptorCreator> _wrappers = [];

  List<InterceptorCreator> get wrappers => new List.unmodifiable(_wrappers);

  final exceptionHandlers = <ExceptionHandler>[];

  bool _wrapsFinalized = false;

  MuxBuilder wrap(InterceptorCreator interceptor) {
    if (_wrapsFinalized) {
      throw new Exception(
          'All interceptors must be added before adding routes or groups!');
    }
    if (interceptor == null) return this;
    _wrappers.add(interceptor);
    return this;
  }

  MuxBuilder onException(ExceptionHandler exceptionHandler) {
    if (_wrapsFinalized) {
      throw new Exception(
          'All interceptors must be added before adding routes or groups!');
    }
    if (exceptionHandler == null) return this;
    exceptionHandlers.add(exceptionHandler);
    return this;
  }

  Route route(String path, Function handler,
      {Map<String, String> pathRegEx,
      List<String> methods: const <String>['GET', 'PUT', 'POST', 'DELETE']}) {
    Route route =
        new Route(path, handler, pathRegEx: pathRegEx, methods: methods);
    route.wrapAll(_wrappers);
    route.onExceptionAll(exceptionHandlers);
    _routes.add(route);
    _wrapsFinalized = true;
    return route;
  }

  Route get(String path, Function handler, {Map<String, String> pathRegEx}) {
    Route route = new Route.Get(path, handler, pathRegEx: pathRegEx);
    route.wrapAll(_wrappers);
    route.onExceptionAll(exceptionHandlers);
    _routes.add(route);
    _wrapsFinalized = true;
    return route;
  }

  Route post(String path, Function handler, {Map<String, String> pathRegEx}) {
    Route route = new Route.Post(path, handler, pathRegEx: pathRegEx);
    route.wrapAll(_wrappers);
    route.onExceptionAll(exceptionHandlers);
    _routes.add(route);
    _wrapsFinalized = true;
    return route;
  }

  Route put(String path, Function handler, {Map<String, String> pathRegEx}) {
    Route route = new Route.Put(path, handler, pathRegEx: pathRegEx);
    route.wrapAll(_wrappers);
    route.onExceptionAll(exceptionHandlers);
    _routes.add(route);
    _wrapsFinalized = true;
    return route;
  }

  Route delete(String path, Function handler, {Map<String, String> pathRegEx}) {
    Route route = new Route.Delete(path, handler, pathRegEx: pathRegEx);
    route.wrapAll(_wrappers);
    route.onExceptionAll(exceptionHandlers);
    _routes.add(route);
    _wrapsFinalized = true;
    return route;
  }

  Group group([String path = '']) {
    return new Group(this, path: path);
  }

  Route addRoute(Route route) {
    _routes.add(route);
    route._wrapAllAtBeginning(_wrappers);
    _wrapsFinalized = true;
    return route;
  }

  Mux build() => new Mux.build(this);
}
