part of jaguar_mux.jaguar_mux;

class Route {
  final String path;

  final List<String> methods;

  final Map<String, String> pathRegEx;

  final Function handler;

  Route(this.path, this.handler,
      {this.methods: const <String>['GET', 'PUT', 'POST', 'DELETE'],
      this.pathRegEx});

  Route.Get(this.path, this.handler, {this.pathRegEx}) : methods = ['GET'];

  Route.Post(this.path, this.handler, {this.pathRegEx}) : methods = ['POST'];

  Route.Put(this.path, this.handler, {this.pathRegEx}) : methods = ['PUT'];

  Route.Delete(this.path, this.handler, {this.pathRegEx})
      : methods = ['DELETE'];

  final List<InterceptorCreator> interceptors = <InterceptorCreator>[];

  final exceptionHandlers = <ExceptionHandler>[];

  Route wrap(InterceptorCreator interceptor) {
    if (interceptor == null) return this;
    interceptors.add(interceptor);
    return this;
  }

  Route wrapAll(List<InterceptorCreator> interceptors) {
    for (InterceptorCreator wrapper in interceptors) {
      wrap(wrapper);
    }
    return this;
  }

  void _wrapAllAtBeginning(List<InterceptorCreator> interceptors) {
    this.interceptors.insertAll(0, interceptors);
  }

  Route onException(ExceptionHandler exceptionHandler) {
    exceptionHandlers.add(exceptionHandler);
    return this;
  }

  Route onExceptionAll(List<ExceptionHandler> exceptions) {
    exceptionHandlers.addAll(exceptions);
    return this;
  }
}
